'File/memory funcs, requires : kernel32 - flags_macros.def.x
'
' #####################
' #####  IO_Init  #####
' #####################
'
FUNCTION IO_Init ()
SHARED IOctrl IOcontrol

IF FLAG_TESTS(IOcontrol,$$IO_INIT) THEN RETURN -1

features=GetCpuFeatures()

IF FLAG_TEST(features,$$CPUID_MMX) THEN FLAG_SETS(IOcontrol,$$IO_MMX)
IF FLAG_TEST(features,$$CPUID_SSE) THEN FLAG_SETS(IOcontrol,$$IO_SSE)
	

FLAG_SETS(IOcontrol,$$IO_INIT)
END FUNCTION
'
' #####################
' #####  IO_Exit  #####
' #####################
'
FUNCTION IO_Exit ()
SHARED IOctrl IOcontrol

FLAG_CLEARS(IOcontrol,$$IO_INIT)

END FUNCTION
'
' ############################
' #####  GetCpuFeatures  #####
' ############################
'
FUNCTION GetCpuFeatures ()
AUTOX ULONG	features

ASM	mov EAX,1
ASM cpuid
ASM mov [GetCpuFeatures.features],EDX

RETURN features
END FUNCTION
'
' ###################
' #####  mCopy  #####
' ###################
'
'performs a copy (len)bytes from (org) to (dest) using MMX if avaible
'28/08/2008 : rewritten to use fully MMX registers (64bytes blocks) because 8 by 8 was not satifaisant
'
FUNCTION mCopy (dest,org,len)
SHARED IOctrl IOcontrol
AUTOX ULONG nQwords,nDwords,nBytes

IF FLAG_TESTS(IOcontrol,$$IO_MMX) THEN
	'mmx copy : full usage of MMX register, 64 bytes each time
	'
	nQwords=len\64
	len=len MOD 64 'adjust len to fit
	
ASM emms	;cleanup MMX state before doing anything
ASM mov ESI,[mCopy.org] ;get function parameters on the stack
ASM mov EDI,[mCopy.dest]
ASM mov EAX,64 ;we will perform 64 by 64 bytes
ASM mov ECX,[mCopy.nQwords]
ASM or ECX,ECX	;check if we have Qwords to copy
ASM jz >.mCopyNoQwords
'the mmx copying loop
ASM .mCopyQwordsLoop
ASM movq MM0,[ESI]
ASM movq MM1,[ESI+8]
ASM movq MM2,[ESI+16]
ASM movq MM3,[ESI+24]
ASM movq MM4,[ESI+32]
ASM movq MM5,[ESI+40]
ASM movq MM6,[ESI+48]
ASM movq MM7,[ESI+56]
ASM add ESI,EAX ;increment source
ASM movq [EDI],MM0
ASM movq [EDI+8],MM1
ASM movq [EDI+16],MM2
ASM movq [EDI+24],MM3
ASM movq [EDI+32],MM4
ASM movq [EDI+40],MM5
ASM movq [EDI+48],MM6
ASM movq [EDI+56],MM7
ASM add EDI,EAX ;increment destination
ASM loop .mCopyQwordsLoop
'bytes copying
ASM .mCopyNoQwords
ASM emms ;cleanup MMX state before exit
ASM jmp >mCopyFromMMX	

ELSE	
	
	
ASM mov ESI,[mCopy.org] ;get function parameters on the stack
ASM mov EDI,[mCopy.dest]	
ASM mCopyFromMMX:
'non mmx : copy 4 by 4 and finish with 1 by 1
nDwords=len\4
nBytes=len MOD 4
	
ASM cld ;be sure movs(d/b) goes forward	
'dwords copying	
ASM	 mov ECX,[mCopy.nDwords]
ASM  rep movsd
'bytes copying
ASM  mov ECX,[mCopy.nBytes]
ASM  rep movsb	

ENDIF
END FUNCTION
'
' ###################
' #####  mFill  #####
' ###################
'
'performs a fill (len)bytes from (org) with DWORD (value) using MMX if avaible
'remarks : if (len MOD 4(8 with MMX))<>0 , last bytes are filled with LSByte of value
'WARNING : This function is well suited to fill memory zones with byte(0), avoid float fill since (len MOD 8)<>0
'or you take risk to have FPU errors when loading a float at end of the memory zone
'for float fill, a simple trick is : write a 2 floats or a double and copy zone on itself with 8 bytes offset
'
FUNCTION mFill (org,len,valueD)
SHARED IOctrl IOcontrol
AUTOX ULONG nQwords,nDwords,nBytes
AUTOX GIANT valueQ

IF FLAG_TESTS(IOcontrol,$$IO_MMX) THEN
	'mmx fill : copy 8 by 8 
	nQwords=len\8
	nBytes=len MOD 8	
	valueQ=GMAKE(valueD,valueD) 
	
ASM	emms
ASM	mov EDI,[mFill.org]
ASM	mov ECX,[mFill.nQwords] 
ASM	or ECX,ECX	;checks we have Qwords to fill
ASM	jz >.mFillNoQwords
ASM	mov EAX,8	;we will perform 8 by 8
ASM movq MM0,[mFill.valueQ] ;load fill value in MM0
ASM .mFillQwordsLoop
ASM movq [EDI],MM0
ASM add EDI,EAX
ASM loop .mFillQwordsLoop
ASM .mFillNoQwords	
ASM emms

ELSE
	'non mmx : copy 4 by 4 
	nDwords=len\4
	nBytes=len MOD 4
	
ASM mov EDI,[mFill.org]	
ASM	mov ECX,[mFill.nDwords] 
ASM	or ECX,ECX	;checks we have Dwords to fill 	
ASM jz >.mFillNoDwords
ASM mov EBX,4	  ;we will perform 4 by 4 ;(use EBX this time cause EAX is used to keep fill value)
ASM mov EAX,[mFill.valueD]
ASM .mFillDwordsLoop
ASM mov [EDI],EAX
ASM add EDI,EBX
ASM loop .mFillDwordsLoop
ASM .mFillNoDwords	
	
ENDIF

'finish 1 by 1
'copying bytes is the same routine with MMX/nonMMX
ASM mov ECX,[mFill.nBytes]
ASM or ECX,ECX ;checks we have bytes to fill	
ASM jz >.mFillNoBytes
ASM mov AL,[mFill.valueD] ;load fill value in AL
ASM .mFillBytesLoop
ASM mov [EDI],AL
ASM inc EDI
ASM loop .mFillBytesLoop
ASM .mFillNoBytes


END FUNCTION
'
' ######################
' #####  mCompare  #####
' ######################
'
'performs comparaisons between (org) and (dest) with the same (len), mmx(uses sse integer) is used if avaible
'returns null if differents
'
FUNCTION mCompare (org,dest,len)
SHARED IOctrl IOcontrol
AUTOX ULONG nQwords,nDwords,nBytes
AUTOX ULONG retval

IF FLAG_TESTS(IOcontrol,($$IO_MMX OR $$IO_SSE)) THEN
	'mmx/sse compare : compare 8 by 8 
	nQwords=len\8
	nBytes=len MOD 8	
	
ASM emms	
ASM mov ESI,[mCompare.org]
ASM mov EDI,[mCompare.dest]
ASM mov ECX,[mCompare.nQwords]
ASM or ECX,ECX
ASM jz >mCompareNoQwords
ASM mov EBX,8 ;we will work 8 by 8
ASM mCompareQwordsLoop:
ASM movq MM0,[ESI]	;loads values to compare
ASM movq MM1,[EDI]
ASM add ESI,EBX
ASM add EDI,EBX
ASM pcmpeqb MM0,MM1 ;compare MM0/MM1, result in MM0
ASM pmovmskb EAX,MM0 ;mov MSB of each MM0 byte in AL (this one is SSE integer)
ASM or EAX,EAX
ASM jz long >mCompareNotEqualMMX
ASM dec ECX
ASM jnz mCompareQwordsLoop

ASM mCompareNoQwords:
ASM emms
	
ELSE
	'non mmx : compare 4 by 4 
	nDwords=len\4
	nBytes=len MOD 4
ASM cld	;be sure CMPSD will go forward 
ASM mov ESI,[mCompare.org]
ASM mov EDI,[mCompare.dest]
ASM mov ECX,[mCompare.nDwords]
ASM or ECX,ECX
ASM jz >.mCompareNoDwords
ASM .mCompareDwords
ASM cmpsd 
ASM jnz >mCompareNotEqual
ASM loop .mCompareDwords
ASM .mCompareNoDwords
	
ENDIF	

'finish 1 by 1
'comparing bytes is the same routine with MMX/nonMMX
ASM cld	;be sure CMPSB will go forward 
ASM mov ECX,[mCompare.nBytes]
ASM or ECX,ECX
ASM jz >.mCompareNoBytes
ASM .mCompareBytes
ASM cmpsb
ASM jnz >mCompareNotEqual
ASM loop .mCompareBytes
ASM .mCompareNoBytes

'blocks are equals, returns 1
RETURN 1

'blocks are not equals, returns 0
ASM mCompareNotEqualMMX:
ASM emms
ASM mCompareNotEqual:

END FUNCTION

