'# #############################################################################
'# ComX_access.m4
'# COM access macros for XBlite
'# Import with m4_include(`ComX_access.m4'), remember to enable macros when compiling
'#
'# Note my conventions for the Expands to section:
'# brackets [] denote optionality
'# braces {} denote that the contents should be a macro
'#
'# COMX macros use the following variables:
'# COMX_temp					You may need to declare this manually if you use the EXPLICIT keyword
'# COMX_TYPE_<type>		where type is some interface structure
'# hr									Return value from CC.  CC only works for COM methods with 4 byte returns
'#
'# #############################################################################

'# #############################################################################
'# FAILED (hr)
'#
'# Returns 0 if hr represents success, otherwise it returns non 0
'# #############################################################################
m4_define(`FAILED', `(($1) AND 0x80000000)')

'# #############################################################################
'# COMOBJ (type, name)
'#
'# Use at start of function with other declarations
'# can be prefixed with SHARED, STATIC or other scope identifier as necassary
'#
'# Expands to:
'#  XLONG name\n
'#  [type COMX_TYPE_type]
'#
'# Defines sub macros:
'#  COMX_TYPEDEF_type		(used to detect whether or not COMX_TYPE_type has been declared)
'#	name_TYPE						(used to detect the type for a COM call)
'# #############################################################################

m4_define(`COMOBJ', `	XLONG $2
	m4_ifdef(`COMX_TYPEDEF_$1', `', `$1 COMX_TYPE_$1
m4_define(`COMX_TYPEDEF_$1', `YES')m4_dnl
m4_define(`$2_TYPE',`COMX_TYPE_$1')m4_dnl')
m4_dnl')

'# #############################################################################
'# CC (obj::method, p1, p2, ... pn-1, pn)
'#
'# Use to call a COM method
'# Return value is stored in variable hr
'#
'# Expands to:
'#  COMX_temp = &{obj_TYPE}
'#  XLONGAT(&&{obj_TYPE}) = XLONGAT(obj)
'#  hr = @obj_TYPE.method(obj, p1, p2, ... pn-1, pn)
'#  XLONGAT(&&{obj_TYPE}) = COMX_temp
'#
'# Defines sub macros:
'#  _COMX_OBJ
'#  _COMX_METHOD
'#  _COMX_OBJTYPE
'# #############################################################################

m4_define(`CC',`m4_define(`_COMX_OBJ',`m4_substr($1, 0, m4_index($1,`::'))')m4_dnl
m4_define(`_COMX_METHOD', `m4_substr($1, m4_eval(m4_index($1,`::')+2))')m4_dnl
m4_define(`_COMX_OBJTYPE', _COMX_OBJ`_TYPE')m4_dnl
	''` $*
	COMX_temp = &`'_COMX_OBJTYPE
	XLONGAT(&&`'_COMX_OBJTYPE) = XLONGAT(_COMX_OBJ)
	hr = @`'_COMX_OBJTYPE`.'_COMX_METHOD`'(_COMX_OBJ m4_ifelse($#, 1, ,`,') m4_shift($*))
	XLONGAT(&&`'_COMX_OBJTYPE) = COMX_temp
m4_dnl')

'# #############################################################################
'# DECLARE_EXTERN_IMPORTERS (v1, v2, ... vn-1, vn)
'#
'# Declares a function to import an external variable
'#
'# Expands to:
'#  DECLARE FUNCTION READ_v1 ()
'#  DECLARE FUNCTION READ_v2 ()
'#	...
'#  DECLARE FUNCTION READ_vn-1 ()
'#  DECLARE FUNCTION READ_vn ()
'# #############################################################################
m4_define(`DECLARE_EXTERN_IMPORTERS', `m4_ifelse($#, 1, `DECLARE FUNCTION READ_$1 ()', `DECLARE FUNCTION READ_$1 ()
DECLARE_EXTERN_IMPORTERS(m4_shift($@))')')

'# #############################################################################
'# DEFINE_EXTERN_IMPORTERS (v1, v2, ... vn-1, vn)
'#
'# Defines a function to import an external variable
'#
'# Expands to:
'#  FUNCTION READ_v1 ()
'#  	XLONG pExtern
'#  ASM mov [READ_v1.pExtern], _v1
'#  	RETURN pExtern
'#  END FUNCTION
'#  FUNCTION READ_v2 ()
'#  	XLONG pExtern
'#  ASM mov [READ_v1.pExtern], _v2
'#  	RETURN pExtern
'#  END FUNCTION
'#  ...
'# #############################################################################
m4_define(`DEFINE_EXTERN_IMPORTERS', `m4_ifelse($#, 1, `FUNCTION READ_$1 ()
	XLONG pExtern
ASM mov eax, ADDR _$1
ASM mov [READ_`'$1`'.pExtern], eax
	RETURN pExtern
END FUNCTION', `FUNCTION READ_$1 ()
	XLONG pExtern
ASM mov eax, ADDR _$1
ASM mov [READ_`'$1`'.pExtern], eax
	RETURN pExtern
END FUNCTION
DEFINE_EXTERN_IMPORTERS(m4_shift($@))')')

'# #############################################################################
'# Tests
'# COMOBJ(IExample, pExample)
'# ###
'# CC(pExample::SomeMethod, p1, p2, p3)
'# ###
'# DECLARE_EXTERN_IMPORTER(IID_IExample)
'# ###
'# DEFINE_EXTERN_IMPORTER(IID_IExample)
